/*-
 * #START_LICENSE#
 * smp-server-library
 * %%
 * Copyright (C) 2017 - 2024 European Commission | eDelivery | DomiSMP
 * %%
 * Licensed under the EUPL, Version 1.2 or – as soon they will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * 
 * [PROJECT_HOME]\license\eupl-1.2\license.txt or https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the Licence is
 * distributed on an "AS IS" basis, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and limitations under the Licence.
 * #END_LICENSE#
 */
package eu.europa.ec.edelivery.smp.data.dao;

import eu.europa.ec.edelivery.smp.data.enums.MembershipRoleType;
import eu.europa.ec.edelivery.smp.data.model.DBDomain;
import eu.europa.ec.edelivery.smp.data.model.user.DBDomainMember;
import eu.europa.ec.edelivery.smp.data.model.user.DBUser;
import org.apache.commons.lang3.StringUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.Collections;
import java.util.List;

import static org.junit.jupiter.api.Assertions.*;

/**
 * @author Joze Rihtarsic
 * @since 5.0
 */
class DomainMemberDaoTest extends AbstractBaseDao {

    @Autowired
    UserDao userDao;
    @Autowired
    DomainDao domainDao;
    @Autowired
    DomainMemberDao testInstance;

    @BeforeEach
    public void prepareDatabase() {
        testUtilsDao.clearData();
        testUtilsDao.createUsers();
        testUtilsDao.createDomains();
    }


    @Test
    void testIsUserDomainsMember() {
        DBDomain domain = testUtilsDao.getD1();
        DBUser user = testUtilsDao.getUser1();
        addMemberToDomain(user, domain, MembershipRoleType.ADMIN);
        // then
        boolean result = testInstance.isUserDomainsMember(user, Collections.singletonList(domain));

        assertTrue(result);
    }

    @Test
    void testIsUserDomainsMemberFalse() {

        // then
        boolean result = testInstance.isUserDomainsMember(testUtilsDao.getUser1(), Collections.singletonList(testUtilsDao.getD1()));

        assertFalse(result);
    }

    @Test
    void testIsUserDomainsMemberWithRoleTrue() {
        DBDomain domain = testUtilsDao.getD1();
        DBUser user = testUtilsDao.getUser1();
        addMemberToDomain(user, domain, MembershipRoleType.ADMIN);
        // then
        boolean result = testInstance.isUserDomainMemberWithRole(user.getId(), Collections.singletonList(domain.getId()), MembershipRoleType.ADMIN);
        assertTrue(result);
        result = testInstance.isUserDomainMemberWithRole(user.getId(), Collections.singletonList(domain.getId()), MembershipRoleType.VIEWER);
        assertFalse(result);
    }

    @Test
    void testGetDomainMembersEmpty() {
        DBDomain domain = testUtilsDao.getD1();
        // then
        Long resultCount = testInstance.getDomainMemberCount(domain.getId(), null);
        List<DBDomainMember> result = testInstance.getDomainMembers(domain.getId(), 0, 10, null);
        assertEquals(0, resultCount.intValue());
        assertEquals(0, result.size());
    }

    @Test
    void testGetDomainMembersOne() {
        DBDomain domain = testUtilsDao.getD1();
        DBUser user = testUtilsDao.getUser1();
        addMemberToDomain(user, domain, MembershipRoleType.ADMIN);
        // then
        Long resultCount = testInstance.getDomainMemberCount(domain.getId(), null);
        List<DBDomainMember> result = testInstance.getDomainMembers(domain.getId(), 0, 10, null);
        assertEquals(1, resultCount.intValue());
        assertEquals(1, result.size());
    }

    @Test
    void testGetDomainMembersOneFilter() {
        DBDomain domain = testUtilsDao.getD1();
        DBUser user = testUtilsDao.getUser1();
        addMemberToDomain(user, domain, MembershipRoleType.ADMIN);
        // then filter no match
        assertFilter("NotExistsAtAll", 0, domain);
        assertFilter(user.getUsername(), 1, domain);
        assertFilter(user.getFullName(), 1, domain);

        assertFilter(StringUtils.upperCase(user.getUsername()), 1, domain);
        assertFilter(StringUtils.upperCase(user.getFullName()), 1, domain);
        assertFilter(StringUtils.lowerCase(user.getUsername()), 1, domain);
        assertFilter(StringUtils.lowerCase(user.getFullName()), 1, domain);
        assertFilter("", 1, domain);
        assertFilter(null, 1, domain);
    }

    private void assertFilter(String filter, int expectedCount, DBDomain domain) {
        Long resultCount = testInstance.getDomainMemberCount(domain.getId(), filter);
        List<DBDomainMember> result = testInstance.getDomainMembers(domain.getId(), 0, 10, filter);
        assertEquals(expectedCount, resultCount.intValue());
        assertEquals(expectedCount, result.size());
    }

    private void addMemberToDomain(DBUser user, DBDomain domain, MembershipRoleType role) {
        DBDomainMember domainMember = new DBDomainMember();
        domainMember.setDomain(domain);
        domainMember.setUser(user);
        domainMember.setRole(role);
        testInstance.persistFlushDetach(domainMember);
    }
}
